import unittest
import logging
import thespian.test.helpers
from thespian.actors import *
import sys
from thespian.test import ActorSystemTestCase


import datetime,os
def mlog(msg):
    with open('/tmp/yorick.log','a') as of:
        of.write('%s %s %s\n'%(datetime.datetime.now(), os.getpid(), str(msg)))


class Yorick(Actor): pass

class Hamlet(Actor):
    def __init__(self, *args, **kw):
        self.numCreated = 0
        self.numDied    = 0
        super(Hamlet, self).__init__(*args, **kw)
    def receiveMessage(self, msg, sender):
        if msg == 'Alas, poor Yorick!':
            self.send(self.myAddress, (sender, 'To be or not to be?'))
        elif isinstance(msg, tuple) and msg[1] == 'To be or not to be?':
            self.send(msg[0], ('That is the question.', self.numCreated, self.numDied))
        elif msg == 'Thyself, yet again':
            self.numCreated += 1
            self.send(sender, self.createActor(Hamlet))
        elif msg == 'Goodbye':
            # Simulate external agent killing an Actor (other than receipt of an ActorExitRequest)
            sys.exit()
        elif isinstance(msg, ActorExitRequest):
            pass
        elif isinstance(msg, ChildActorExited):
            self.numDied += 1
        else:
            mlog('Unknown message: %s'%str(msg))
            raise RuntimeError('unknown message: %s'%str(msg))


class DLA(Actor):
    def __init__(self, *args, **kw):
        self.numHandled = 0
        super(DLA, self).__init__(*args, **kw)
    def receiveMessage(self, msg, sender):
        if msg == 'Begin':
            self.handleDeadLetters()
        elif isinstance(msg, ActorExitRequest):
            pass
        elif msg == 'Count?':
            self.send(sender, self.numHandled)
        else:
            self.numHandled += 1


#until multiprocess system works on Windows
import os
if os.name!='nt':
    class TestMultiProcessSystem(ActorSystemTestCase):

        def setUp(self):
            self.setSystemBase('multiprocessSystemBase')

        def testSubActorIsFunctional(self):
            aS = ActorSystem()
            hamlet = aS.createActor(Hamlet)
            again = aS.ask(hamlet, 'Thyself, yet again')
            self.assertEqual(aS.ask(hamlet, 'Alas, poor Yorick!', 1),
                             ('That is the question.', 1, 0))
            self.assertEqual(aS.ask(again, 'Alas, poor Yorick!', 1),
                             ('That is the question.', 0, 0))

        def testSubActorKill(self):
            aS = ActorSystem()
            dla = aS.createActor(DLA)
            aS.tell(dla, 'Begin')
            hamlet = aS.createActor(Hamlet)
            again = aS.ask(hamlet, 'Thyself, yet again')
            self.assertEqual(aS.ask(dla, 'Count?', 1), 0)
            aS.tell(again, 'Goodbye')
            self.assertEqual(aS.ask(hamlet, 'Alas, poor Yorick!', 1),
                             ('That is the question.', 1, 1))
            self.assertEqual(aS.ask(dla, 'Count?', 1), 0)
            #aS.tell(again, 'Speak')
            self.assertEqual(aS.ask(dla, 'Count?', 1), 2)

        def testSub2ActorKill(self):
            aS = ActorSystem()
            hamlet = aS.createActor(Hamlet)
            again = aS.ask(hamlet, 'Thyself, yet again')
            again = aS.ask(hamlet, 'Thyself, yet again')
            aS.tell(again, 'Goodbye')
            self.assertEqual(aS.ask(hamlet, 'Alas, poor Yorick!', 1),
                             ('That is the question.', 2, 1))

        def testSubSubActorKill(self):
            aS = ActorSystem()
            hamlet = aS.createActor(Hamlet)
            mid = aS.ask(hamlet, 'Thyself, yet again')
            again = aS.ask(mid, 'Thyself, yet again')
            aS.tell(again, 'Goodbye')
            self.assertEqual(aS.ask(hamlet, 'Alas, poor Yorick!', 1),
                             ('That is the question.', 1, 0))
            self.assertEqual(aS.ask(mid, 'Alas, poor Yorick!', 1),
                             ('That is the question.', 1, 1))

        def testMiddleActorKill(self):
            aS = ActorSystem()
            hamlet = aS.createActor(Hamlet)
            mid = aS.ask(hamlet, 'Thyself, yet again')
            again = aS.ask(mid, 'Thyself, yet again')
            aS.tell(mid, 'Goodbye')
            self.assertEqual(aS.ask(hamlet, 'Alas, poor Yorick!', 1),
                             ('That is the question.', 1, 1))
            # Next shows that sub-actor of killed actor is untouched and survives.
            self.assertEqual(aS.ask(again, 'Alas, poor Yorick!', 1),
                             ('That is the question.', 0, 0))

        def testTopActorKill(self):
            aS = ActorSystem()
            hamlet = aS.createActor(Hamlet)
            mid = aS.ask(hamlet, 'Thyself, yet again')
            again = aS.ask(mid, 'Thyself, yet again')
            aS.tell(hamlet, 'Goodbye')
            self.assertEqual(aS.ask(mid, 'Alas, poor Yorick!', 1),
                             ('That is the question.', 1, 0))
            # Next shows that sub-actor of killed actor is untouched and survives.
            self.assertEqual(aS.ask(again, 'Alas, poor Yorick!', 1),
                             ('That is the question.', 0, 0))
